#include "sdl.h"
#include <vector>
#include <cstdio>
#include <cstdlib>

// Factor de conversion grados a radianes:
static constexpr float pi= 3.1416;
static constexpr float factor= pi/180.0;
// Gráfico de nave, 5 puntos, 4 segmentos:
static const std::vector<sdl::FPoint> nave = { {15.0, 0.0}, {-15.0, -9.0}, {-11.0, 0.0}, {-15.0, 9.0}, {15.0, 0.0} };
static const std::vector<sdl::FPoint> estela = { {-16.0, -4.0}, {-25.0, 0.0}, {-16.0, 4.0} };

class Nave {
private:
    std::vector<sdl::FPoint> nave2;
    std::vector<sdl::FPoint> estela2;
    sdl::FPoint posicion;
    sdl::FPoint velocidad;
    void trasladar(float alfa, sdl::FPoint &desplazamiento, sdl::FPoint &escala) {
        for(size_t i = 0; i < nave.size(); i++) {
            nave2[i] = nave[i].Move(alfa, desplazamiento, escala);
        }
        for(size_t i = 0; i < estela.size(); i++) {
            estela2[i] = estela[i].Move(alfa, desplazamiento, escala);
        }
    }

public:
    Nave(const sdl::FPoint &pos, const float alfa) : posicion(pos), velocidad(0.0, 0.0) {
        nave2.resize(nave.size());
        estela2.resize(estela.size());
    }
    void Actualizar(Uint64 tick, float acelerar, float alfa, sdl::FPoint escala) {
        velocidad.X() += acelerar*cos(factor*alfa);
        velocidad.Y() += -acelerar*sin(factor*alfa);
        posicion.X() += (velocidad.X()*(tick)/1000.0);
        posicion.Y() += (velocidad.Y()*(tick)/1000.0);
        if(posicion.X()>635) posicion.X() -=640;
        if(posicion.X()<5) posicion.X() +=640;
        if(posicion.Y()>475) posicion.Y() -=480;
        if(posicion.Y()<5) posicion.Y() +=480;
        // Trasladar nave:
        trasladar(factor*alfa, posicion, escala);
    }
    std::vector<sdl::FPoint> &PuntosNave() { return nave2; }
    std::vector<sdl::FPoint> &PuntosEstela() { return estela2; }
    sdl::FPoint Posicion() { return posicion; }
    //int nPuntos() const { return npNave; }
};

class Asteroides : public sdl::Game {
private:
    sdl::FPoint escala; // Escala del juego (ajustada a una resolución de 640x480
    Nave nave;
    float acelerar;
    float alfa;

public:
    Asteroides(sdl::Window& window, sdl::Renderer& renderer) :
      sdl::Game(window, renderer),
      nave({320.0, 240.0}, 0.0),
      acelerar(0.0), alfa(0.0) {
        int w, h;
        window.GetSize(&w, &h);
        escala = sdl::FPoint(w/640.0, h/480.0);
    }
    const sdl::FPoint &Escala() { return escala; }
    void Init();
    void Events();
    void Update();
    void Render();
};

void Asteroides::Events() {
    static bool izq;
    static bool der;
    static bool arr;

    while(event.Poll()) {
        switch(event.Type()) {
        case SDL_KEYDOWN:
            if(event.Key().keysym.sym == SDLK_LEFT)  { izq = true; }
            if(event.Key().keysym.sym == SDLK_RIGHT) { der = true; }
            if(event.Key().keysym.sym == SDLK_UP)    { arr = true; }
            if(event.Key().keysym.sym == SDLK_ESCAPE) {
                quit = true; }
            break;
        case SDL_KEYUP:
            if(event.Key().keysym.sym == SDLK_LEFT)  { izq = false; }
            if(event.Key().keysym.sym == SDLK_RIGHT) { der = false; }
            if(event.Key().keysym.sym == SDLK_UP)    { arr = false; }
            break;
        }
    }
    acelerar = 0.0;
    if(izq && !der) alfa -= tick/10.0;
    if(!izq && der) alfa += tick/10.0;
    if(alfa < 0) alfa+=360.0;
    if(alfa >= 360.0) alfa-=360.0;
    if(arr) acelerar = tick/15.0;
}

void Asteroides::Render() {
    // Renderizado:
    sdl::Color negro("#000000ff");
    sdl::Color blanco("#ffffffff");
    sdl::Color amarillo("#ffff00ff");

    renderer.SetDrawColor(negro);
    renderer.Clear();
    renderer.SetDrawColor(blanco);
    renderer.DrawLinesF(nave.PuntosNave());
    if(acelerar != 0.0) {
        renderer.SetDrawColor(amarillo);
        renderer.DrawLinesF(nave.PuntosEstela());
    }
    renderer.Present();
}

void Asteroides::Update() {
    nave.Actualizar(tick, acelerar, alfa, escala);
}

void Asteroides::Init() {}

int main( int argc, char * argv[] ) {
    sdl::SDL2 sdl(SDL_INIT_EVERYTHING);
    sdl::Window window ("Asteroides", 800, 600, (SDL_WindowFlags)(SDL_WINDOW_SHOWN | SDL_WINDOW_BORDERLESS));
    sdl::Renderer renderer(window, SDL_RENDERER_ACCELERATED);

    Asteroides asteroides(window, renderer);

    asteroides.Run();
    return 0;
}
